#!/usr/bin/env python
#
# bhv - applyTagDialog.py
# Author: Loren Bandiera
# Copyright 2005, 2006 MMG Security, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#

import sys
import gtk
import gtk.glade

from config import *
from pysqlite2 import dbapi2 as sqlite

class ApplyTagDialog:
	def __init__ (self, db_connection):
		self.db_con = db_connection
		self.user_tags = []

		self.glade_file = os.path.join (GLADEDIR, "browser-history-viewer.glade")
		self.Glade = gtk.glade.XML (self.glade_file, "applyTagDialog")

		self.tagDialog = self.Glade.get_widget ("applyTagDialog")
		self.tag_treeview = self.Glade.get_widget ("tagTreeView")
		self.tag_store = gtk.ListStore (bool, str, str)
		self.tag_treeview.set_model (self.tag_store)

		cell = gtk.CellRendererToggle()
		cell.set_property ("activatable", True)
		cell.connect ("toggled", self.on_tag_toggled)

		hc1 = gtk.TreeViewColumn('Enabled', cell, active=0)
		hc1.set_resizable (True)
		hc1.set_spacing (5)
		self.tag_treeview.append_column (hc1)

		hc1 = self.create_tree_column ('Name', 1)
		self.tag_treeview.append_column (hc1)

		hc2 = self.create_tree_column ('Description', 2)
		self.tag_treeview.append_column (hc2)

		self.get_tags ()

		self.tagDialog.resize (320, 200)
		self.tagDialog.run ()
		self.tagDialog.destroy ()

		return

	def create_tree_column (self, name, pos):
		col = gtk.TreeViewColumn(name, gtk.CellRendererText(), text=pos)
		col.set_resizable (True)
		col.set_spacing (5)
		col.set_sort_column_id (pos)
		return col

	def get_tags (self):
		cur = self.db_con.cursor()
		cmd = "select * from tags"
		cur.execute (cmd)

		for (name,description) in cur:
			self.tag_store.append ([False, name, description])

	def on_tag_toggled (self, crt, path):
		row = self.tag_store.get_iter (path)
		old = self.tag_store.get_value (row, 0)
		name = self.tag_store.get_value (row, 1)

		if (not old):
			self.user_tags.append (name)
		else:
			self.user_tags.remove (name)

		self.tag_store.set_value (row, 0, not old)

	def get_user_tags (self):
		return self.user_tags
