#!/usr/bin/env python
#
# bhv - bhvwindow.py
# Author: Loren Bandiera
# Copyright 2005, 2006 MMG Security, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#

import array
import string
import sys
import os.path
import time
import csv
import gettext
import urlparse
import xml.dom.ext
import xml.dom.minidom
import gtk
import gtk.glade
import gnome
import gnome.ui
import gtkmozembed
import importDialog
import exportDialog
import tagDialog
import applyTagDialog

from config import *
from pysqlite2 import dbapi2 as sqlite

gettext.install (PACKAGE, LOCALEDIR)

class BrowserHistoryWindow:
	def __init__ (self):
		self.gnome_program = gnome.init(PACKAGE, VERSION)
		self.db = None
		self.cache_dir = ""
		self.current_file_type = 0

		self.file_history = []
		self.file_history_index = 0

		self.hv_history = []
		self.hv_history_index = 0

		self.glade_file = os.path.join (GLADEDIR, "browser-history-viewer.glade")
		self.Glade = gtk.glade.XML (self.glade_file, "mainWindow")

		# setup main window
		self.mainwindow = self.Glade.get_widget ("mainWindow")
		self.mainwindow.resize (640, 480)

		self.notebook = self.Glade.get_widget ("notebook1");

		self.hpaned = self.Glade.get_widget ("hpaned1")
		self.hpaned.set_position (250)

		self.vpaned = self.Glade.get_widget ("vpaned1")
		self.vpaned.set_position (150)

		export_button = self.Glade.get_widget ("exportMenuToolButton")
		export_menu = gtk.Menu ()
		
		all_menu_item = gtk.MenuItem (label="Export all...")
		all_menu_item.connect ("activate", self.on_export_all_activate)
		all_menu_item.show ()
		export_menu.append (all_menu_item)

		by_host_item = gtk.MenuItem (label="Export by host...")
		by_host_item.connect ("activate", self.on_export_by_host_activate)
		by_host_item.show ()
		export_menu.append (by_host_item)

		by_tag_item = gtk.MenuItem (label="Export by tag...")
		by_tag_item.connect ("activate", self.on_export_by_tag_activate)
		by_tag_item.show ()
		export_menu.append (by_tag_item)

		export_button.set_menu (export_menu)

		self.toggle_toolbar (False)
		
		# sources treeview
		self.history_sources_treeview = self.Glade.get_widget ("historySourcesTreeView") 
		self.history_sources_store = gtk.TreeStore (str, int)
		self.history_sources_treeview.set_model (self.history_sources_store)

		hc1 = self.create_tree_column ('Name', 0)
		self.history_sources_treeview.append_column (hc1)

		hc2 = self.create_tree_column ('Type', 1)
		hc2.set_visible (False)
		self.history_sources_treeview.append_column (hc2)

		self.history_sources_store.append (None, ['All history', 0])
		self.history_sources_treeview.set_sensitive (False)

		# data treeview
		self.history_data_treeview = self.Glade.get_widget ("historyDataTreeView") 
		self.history_data_store = gtk.ListStore (int, str, str, str, str, str, str, str, str)
		self.history_data_treeview.set_model (self.history_data_store)
		self.history_data_treeview.get_selection().set_mode (gtk.SELECTION_MULTIPLE)

		self.data_col_record_num = self.create_tree_column ('Record #', 0)
		self.history_data_treeview.append_column (self.data_col_record_num)

		self.data_col_cache_filename = self.create_tree_column ('Filename', 1)
		self.history_data_treeview.append_column (self.data_col_cache_filename)

		self.data_col_first_access_time = self.create_tree_column ('First Access Time', 2)
		self.history_data_treeview.append_column (self.data_col_first_access_time)

		self.data_col_last_access_time = self.create_tree_column ('Last Access Time', 3)
		self.history_data_treeview.append_column (self.data_col_last_access_time)

		self.data_col_source = self.create_tree_column ('Source', 4)
		self.history_data_treeview.append_column (self.data_col_source)

		cell = gtk.CellRendererText()
		cell.set_property('editable', True)
		cell.connect("edited", self.on_tags_edited)

		col = gtk.TreeViewColumn('Tags', cell, text=5)
		col.set_resizable (True)
		col.set_spacing (5)
		col.set_sort_column_id (5)

		self.data_col_tags = col
		self.history_data_treeview.append_column (self.data_col_tags)

		self.data_col_title = self.create_tree_column ('Title', 6)
		self.history_data_treeview.append_column (self.data_col_title)

		self.data_col_url = self.create_tree_column ('URL', 7)
		self.history_data_treeview.append_column (self.data_col_url)

		self.data_col_visit_count = self.create_tree_column ('Visit Count', 8)
		self.history_data_treeview.append_column (self.data_col_visit_count)

		data_selection = self.history_data_treeview.get_selection()
		data_selection.connect("changed", self.on_data_selection_changed)

		self.history_data_treeview.connect ("button-press-event", self.on_right_click);

		self.history_data_treeview.set_sensitive (False)

		# gtkmozembed
		gtkmozembed.set_profile_path("/tmp", "bhv")
		self.moz = gtkmozembed.MozEmbed()
		self.moz.connect("net-stop", self.on_moz_stop)
		self.moz.show()
		self.moz.set_sensitive (False)

		self.moz_hbox = self.Glade.get_widget ("mozHBox")
		self.moz_hbox.pack_start (self.moz, True, True, 5);

		# setup file treeview
		self.directory_entry = self.Glade.get_widget ("baseDirectoryEntry")
		self.file_treeview = self.Glade.get_widget ("fileTreeView")

		self.file_store = gtk.ListStore (str)
		self.file_treeview.set_model (self.file_store)

		file_c1 = self.create_tree_column ('Name', 0)
		self.file_treeview.append_column (file_c1)

		file_selection = self.file_treeview.get_selection()
		file_selection.connect("changed", self.on_file_selection_changed)

		# search view
		w = self.Glade.get_widget ("searchComboBox")
		w.set_active (0)
		self.toggle_search_view (False)

		# connect event handlers
		dic = { 
			"OnNewActivate" : self.on_new_activate,
			"OnOpenActivate" : self.on_open_activate,
			"OnImportActivate" : self.on_import_activate,
			"OnExportActivate" : self.on_export_all_activate,
			"OnTagActivate" : self.on_tag_activate,
			"OnTagManagerActivate" : self.on_tag_manager_activate,
			"OnBackActivate" : self.on_back_activate,
			"OnForwardActivate" : self.on_forward_activate,
			"OnCloseActivate" : self.on_close_activate,
			"OnQuitActivate" : (gtk.main_quit),
			"OnPreferencesActivate" : self.on_preferences_activate,
			"OnHistorySourceActivate" : self.on_history_source_activated,
			"OnHistorySourceExpand" : self.on_history_source_expanded,
			"OnBaseDirectoryActivate" : self.on_base_dir_activated,
			"OnFileRowActivate" : self.on_file_row_activated,
			"OnClearSearchClicked" : self.on_clear_search_clicked,
			"OnFindClicked" : self.on_find_clicked,
			"OnAppDelete" : (gtk.main_quit),
			"OnAboutActivate" : self.on_about_activated,
			"OnHelpContentsActivate" : self.on_help_activated }
		
		self.Glade.signal_autoconnect (dic)
		return

	def toggle_search_view (self, state):
		w = self.Glade.get_widget ("searchComboBox")
		w.set_sensitive (state)

		w = self.Glade.get_widget ("searchTextEntry")
		w.set_sensitive (state)

		w = self.Glade.get_widget ("button1")
		w.set_sensitive (state)

		w = self.Glade.get_widget ("button2")
		w.set_sensitive (state)

	
	def toggle_toolbar (self, state):
		w = self.Glade.get_widget ("import_browser_history1")
		w.set_sensitive (state)

		w = self.Glade.get_widget ("save")
		w.set_sensitive (state)

		w = self.Glade.get_widget ("close1")
		w.set_sensitive (state)

		w = self.Glade.get_widget ("importToolButton")
		w.set_sensitive (state)

		w = self.Glade.get_widget ("exportMenuToolButton")
		w.set_sensitive (state)

		w = self.Glade.get_widget ("backToolButton")
		w.set_sensitive (state)

		w = self.Glade.get_widget ("forwardToolButton")
		w.set_sensitive (state)

		w = self.Glade.get_widget ("tagToolButton")
		w.set_sensitive (state)

		w = self.Glade.get_widget ("tagManagerToolButton")
		w.set_sensitive (state)

	def create_tree_column (self, name, pos):
		col = gtk.TreeViewColumn(name, gtk.CellRendererText(), text=pos)
		col.set_resizable (True)
		col.set_spacing (5)
		col.set_sort_column_id (pos)
		return col

	def update_file_list (self, path):
		try:
			contents = os.listdir (path)
			self.file_store.clear ()
			for name in contents:
				self.file_store.append ([name])
			return True
		except:
			return False

	def on_clear_search_clicked (self, widget):
		w = self.Glade.get_widget ("searchTextEntry")
		w.set_text ("")

		self.history_data_store.clear ()

	def on_find_clicked (self, widget):
		w = self.Glade.get_widget ("searchComboBox")
		search_iter = w.get_active_iter ()
		search_model = w.get_model ()
		search_field = string.lower (search_model.get_value (search_iter, 0))

		w = self.Glade.get_widget ("searchTextEntry")
		search_text = w.get_text ()

		cur = self.db_con.cursor()
		search_string = "'%%%s%%'" % search_text
		cmd = "select * from history where " + search_field + " like " + search_string
		cur.execute (cmd)

		self.history_data_store.clear ()

		for (record_num,cache_filename,first_access_time,last_access_time,source,tags,title,url,visit_count) in cur:
			self.history_data_store.append ([record_num,cache_filename,first_access_time,last_access_time,source,tags,title,url,visit_count])

	def on_base_dir_activated (self, widget):
		self.current_dir = self.directory_entry.get_text ()
		self.update_file_list (self.current_dir)
		self.file_history.append (self.current_dir)
		self.file_history_index += 1

	def on_file_row_activated (self, treeview, path, view_column):
		row = self.file_store.get_iter (path)
		name = self.file_store.get_value (row, 0)
		new_path = os.path.join (self.current_dir, name)

		if (os.path.isdir (new_path)):
			self.current_dir = new_path
			self.directory_entry.set_text (self.current_dir)
			self.file_history.append (self.current_dir)
			self.file_history_index += 1
			self.update_file_list (self.current_dir)

	def on_file_selection_changed (self, widget):
		try:
			model, row = self.file_treeview.get_selection().get_selected()		
			name = self.file_store.get_value (row, 0)

			path = os.path.join (self.current_dir, name)

			if (os.path.isfile (path)):
				self.file_history.append (path)
				self.file_history_index += 1
				self.moz.load_url (path)

		except:
			filename = ""

	def on_tags_edited (self, widget, path, new_text):
		old_text = ""
		model, row = self.history_data_treeview.get_selection().get_selected()

		record_num = self.history_data_store.get_value (row, 0)
		old_text = self.history_data_store.get_value (row, 5)

		if (old_text == new_text):
			return

		self.history_data_store.set_value (row, 5, new_text)

		cur = self.db_con.cursor()
		cur.execute ("update history set tags=? where record_num=?", (new_text,record_num))
		self.db_con.commit ()

	def on_moz_stop (self, widget):
		current_page = self.notebook.get_current_page ()
		if (current_page == 0):
			self.history_data_treeview.grab_focus ()
		elif (current_page == 1):
			self.file_treeview.grab_focus ()

	def on_data_selection_changed (self, widget):
		try:
			model, paths = self.history_data_treeview.get_selection().get_selected_rows()
			num_rows = self.history_data_treeview.get_selection().count_selected_rows()

			if (num_rows == 1):
				row = self.history_data_store.get_iter (paths[0])
				uri = self.history_data_store.get_value (row, 7)
				self.hv_history.append (uri)
				self.hv_history_index += 1
				self.moz.load_url (uri)
		except:
			uri = ""

	def show_all_data_columns (self):
		self.data_col_record_num.set_visible (True)
		self.data_col_cache_filename.set_visible (True)
		self.data_col_first_access_time.set_visible (True)
		self.data_col_last_access_time.set_visible (True)
		self.data_col_source.set_visible (True)
		self.data_col_tags.set_visible (True)
		self.data_col_title.set_visible (True)
		self.data_col_url.set_visible (True)
		self.data_col_visit_count.set_visible (True)
		
	def show_mozilla_data_columns (self):
		self.data_col_record_num.set_visible (True)
		self.data_col_cache_filename.set_visible (False)
		self.data_col_first_access_time.set_visible (True)
		self.data_col_last_access_time.set_visible (True)
		self.data_col_source.set_visible (False)
		self.data_col_tags.set_visible (True)
		self.data_col_title.set_visible (False)
		self.data_col_url.set_visible (True)
		self.data_col_visit_count.set_visible (True)

	def show_ie_data_columns (self):
		self.data_col_record_num.set_visible (True)
		self.data_col_cache_filename.set_visible (False)
		self.data_col_first_access_time.set_visible (False)
		self.data_col_last_access_time.set_visible (True)
		self.data_col_source.set_visible (False)
		self.data_col_tags.set_visible (True)
		self.data_col_title.set_visible (False)
		self.data_col_url.set_visible (True)
		self.data_col_visit_count.set_visible (False)

	def show_opera_data_columns (self):
		self.data_col_record_num.set_visible (True)
		self.data_col_cache_filename.set_visible (False)
		self.data_col_first_access_time.set_visible (False)
		self.data_col_last_access_time.set_visible (True)
		self.data_col_source.set_visible (False)
		self.data_col_tags.set_visible (True)
		self.data_col_title.set_visible (True)
		self.data_col_url.set_visible (True)
		self.data_col_visit_count.set_visible (False)

	def show_safari_data_columns (self):
		self.data_col_record_num.set_visible (True)
		self.data_col_cache_filename.set_visible (False)
		self.data_col_first_access_time.set_visible (False)
		self.data_col_last_access_time.set_visible (True)
		self.data_col_source.set_visible (False)
		self.data_col_tags.set_visible (True)
		self.data_col_title.set_visible (True)
		self.data_col_url.set_visible (True)
		self.data_col_visit_count.set_visible (True)

	def show_epiphany_data_columns (self):
		self.data_col_record_num.set_visible (True)
		self.data_col_cache_filename.set_visible (False)
		self.data_col_first_access_time.set_visible (True)
		self.data_col_last_access_time.set_visible (True)
		self.data_col_source.set_visible (False)
		self.data_col_tags.set_visible (True)
		self.data_col_title.set_visible (True)
		self.data_col_url.set_visible (True)
		self.data_col_visit_count.set_visible (True)

	def on_history_source_expanded (self, treeview, treeiter, path):
		source_name = self.history_sources_store.get_value (treeiter, 0)
		source_type = self.history_sources_store.get_value (treeiter, 1)

		child = self.history_sources_store.iter_children (treeiter)
		child_name = self.history_sources_store.get_value (child, 0)

		if (child_name != ""):
			return		

		self.history_sources_store.remove (child)

		cur = self.db_con.cursor()
		cur.execute ("select url from history where source=?", (source_name,))

		hosts = []

		for row in cur:
			data = urlparse.urlparse (row[0])
			try:
				hosts.index (data[1])
			except:
				if (data[1] != ""):
					hosts.append (data[1])

		hosts.sort ()
		for h in hosts:
			self.history_sources_store.append (treeiter, [h, source_type])

	def on_history_source_activated (self, treeview, path, view_column):
		model, row = self.history_sources_treeview.get_selection().get_selected()

		byhost = False
		hostname = ""

		parent = self.history_sources_store.iter_parent (row)
		if (parent == None):
			source_name = self.history_sources_store.get_value (row, 0)
		else:
			source_name = self.history_sources_store.get_value (parent, 0)
			hostname = self.history_sources_store.get_value (row, 0)
			byhost = True

		source_type = self.history_sources_store.get_value (row, 1)

		cmd = ""

		if (source_type == 0):
			cmd = "select * from history"
			self.show_all_data_columns ()
		elif (source_type == 1):
			self.show_mozilla_data_columns ()
		elif (source_type == 2):
			self.show_ie_data_columns ()
		elif (source_type == 3):
			self.show_opera_data_columns ()
		elif (source_type == 4):
			self.show_safari_data_columns ()
		
		cur = self.db_con.cursor()

		if (cmd != ""):
			cur.execute (cmd)
		else:
			if (byhost):
				search_string = "'%%%s%%'" % hostname
				cur.execute ("select * from history where source=? and url like " + search_string, (source_name,))
			else:
				cur.execute ("select * from history where source=?", (source_name,))


		self.history_data_store.clear ()

		for (record_num,cache_filename,first_access_time,last_access_time,source,tags,title,url,visit_count) in cur:
			self.history_data_store.append ([record_num,cache_filename,first_access_time,last_access_time,source,tags,title,url,visit_count])

	def create_db_tables (self):
		cur = self.db_con.cursor()
		cur.executescript ("""
		create table source (
			filename,
			description,
			file_type
		);

		create table tags (
			name,
			description
		);

		create table history (
			record_num,
			cache_filename,
			first_access_time,
			last_access_time,
			source,
			tags,
			title,
			url,
			visit_count
		);
		""")

	def on_new_activate (self, widget):
		dialog = gtk.FileChooserDialog("Save investigation data to...",
		                               None,
		                               gtk.FILE_CHOOSER_ACTION_SAVE,
		                               (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
		                                gtk.STOCK_SAVE, gtk.RESPONSE_OK))

		dialog.set_default_response(gtk.RESPONSE_OK)

		filter = gtk.FileFilter()
		filter.set_name("All files (*.*)")
		filter.add_pattern("*")
		dialog.add_filter(filter)

		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			self.db_filename = dialog.get_filename()

			if os.path.exists (self.db_filename):
				result = message_box(None, 0, gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO, "The file '%s' exists. Do you want to overwrite it?" % self.db_filename)
				if (result == gtk.RESPONSE_YES):
					os.remove (self.db_filename)
				else:
					dialog.destroy ()
					return

			self.db_con = con = sqlite.connect(self.db_filename, check_same_thread=False)
			self.create_db_tables ()

			self.history_sources_treeview.set_sensitive (True)
			self.history_data_treeview.set_sensitive (True)
			self.moz.set_sensitive (True)
			self.toggle_toolbar (True)
			self.toggle_search_view (True)

		dialog.destroy()

	def on_import_activate (self, widget):
		dialog = importDialog.ImportDialog (self.db_con)
		response = dialog.run ()

		if response == gtk.RESPONSE_OK:
			self.history_sources_store.append (None, [dialog.get_file_description(), dialog.get_file_type()])

		dialog.destroy ()

	def on_open_activate (self, widget):
		dialog = gtk.FileChooserDialog("Select investigation to open",
		                               None,
		                               gtk.FILE_CHOOSER_ACTION_OPEN,
		                               (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
		                                gtk.STOCK_OPEN, gtk.RESPONSE_OK))

		dialog.set_default_response(gtk.RESPONSE_OK)

		filter = gtk.FileFilter()
		filter.set_name("All files")
		filter.add_pattern("*")
		dialog.add_filter(filter)

		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			self.db_filename = dialog.get_filename()
			self.db_con = con = sqlite.connect(self.db_filename, check_same_thread=False)

			cur = self.db_con.cursor()
			cmd = "select * from source"
			cur.execute (cmd)

			for (filename,description,file_type) in cur:
				par = self.history_sources_store.append (None, [description, file_type])
				self.history_sources_store.append (par, ["", 0])

			self.history_sources_treeview.set_sensitive (True)
			self.history_data_treeview.set_sensitive (True)
			self.moz.set_sensitive (True)

			self.toggle_toolbar (True)
			self.toggle_search_view (True)

			title = "%s - Browser History Viewer" % self.db_filename
			self.mainwindow.set_title (title)

		dialog.destroy()
	
	def save_csv_history (self, filename):
		writer = csv.writer (open(filename, "wb"))

		cur = self.db_con.cursor()
		cmd = "select * from history"
		cur.execute (cmd)

		for row in cur:
			writer.writerow (row)

	def save_csv_history_by_value (self, filename, value_type, values):
		writer = csv.writer (open(filename, "wb"))
		cur = self.db_con.cursor()

		cmd = ""
		if (value_type == 0):
			cmd = "select * from history where url like "
		elif (value_type == 1):
			cmd = "select * from history where tags like "

		for s in values:
			search_string = "'%%%s%%'" % s
			cur.execute (cmd + search_string)
			for row in cur:
				writer.writerow (row)

	def save_tab_history (self, filename):
		f = open(filename, "wb")

		cur = self.db_con.cursor()
		cmd = "select * from history"
		cur.execute (cmd)

		for (record_num,cache_filename,first_access_time,last_access_time,source,tags,title,url,visit_count) in cur:
			line = "%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n" % (record_num,cache_filename,first_access_time,last_access_time,source,tags,title,url,visit_count)
			f.write (line)			

		f.close ()

	def save_tab_history_by_value (self, filename, value_type, values):
		f = open(filename, "wb")
		cur = self.db_con.cursor()

		cmd = ""
		if (value_type == 0):
			cmd = "select * from history where url like "
		elif (value_type == 1):
			cmd = "select * from history where tags like "

		for s in values:
			search_string = "'%%%s%%'" % s
			cur.execute (cmd + search_string)
			for (record_num,cache_filename,first_access_time,last_access_time,source,tags,title,url,visit_count) in cur:
				line = "%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n" % (record_num,cache_filename,first_access_time,last_access_time,source,tags,title,url,visit_count)
				f.write (line)

		f.close ()

	def save_xml_history (self, filename):
		doc = xml.dom.minidom.Document()

		cur = self.db_con.cursor()
		cmd = "select * from history"
		cur.execute (cmd)

		element_bh = doc.createElement ("browserHistory")

		for (record_num,cache_filename,first_access_time,last_access_time,source,tags,title,url,visit_count) in cur:
			element_bhe = doc.createElement ("browserHistoryEntry")
	
			str_num = "%d" % record_num

			element_rec_num = doc.createElement ("recordNumber")
			rec_num_text = doc.createTextNode (str_num)
			element_rec_num.appendChild (rec_num_text)
			element_bhe.appendChild (element_rec_num)

			element_rec_num = doc.createElement ("cacheFilename")
			rec_num_text = doc.createTextNode (cache_filename)
			element_rec_num.appendChild (rec_num_text)
			element_bhe.appendChild (element_rec_num)

			element_rec_num = doc.createElement ("firstAccessTime")
			rec_num_text = doc.createTextNode (first_access_time)
			element_rec_num.appendChild (rec_num_text)
			element_bhe.appendChild (element_rec_num)

			element_rec_num = doc.createElement ("lastAccessTime")
			rec_num_text = doc.createTextNode (last_access_time)
			element_rec_num.appendChild (rec_num_text)
			element_bhe.appendChild (element_rec_num)

			element_rec_num = doc.createElement ("sourceFile")
			rec_num_text = doc.createTextNode (source)
			element_rec_num.appendChild (rec_num_text)
			element_bhe.appendChild (element_rec_num)

			element_rec_num = doc.createElement ("tags")
			rec_num_text = doc.createTextNode (tags)
			element_rec_num.appendChild (rec_num_text)
			element_bhe.appendChild (element_rec_num)

			element_rec_num = doc.createElement ("title")
			rec_num_text = doc.createTextNode (title)
			element_rec_num.appendChild (rec_num_text)
			element_bhe.appendChild (element_rec_num)

			element_rec_num = doc.createElement ("url")
			rec_num_text = doc.createTextNode (url)
			element_rec_num.appendChild (rec_num_text)
			element_bhe.appendChild (element_rec_num)

			str_num = "%s" % visit_count

			element_rec_num = doc.createElement ("visitCount")
			rec_num_text = doc.createTextNode (str_num)
			element_rec_num.appendChild (rec_num_text)
			element_bhe.appendChild (element_rec_num)

			element_bh.appendChild (element_bhe)

		doc.appendChild (element_bh)

		xml.dom.ext.PrettyPrint(doc, open(filename, "w"))		 

	def save_xml_history_by_value (self, filename, value_type, values):
		doc = xml.dom.minidom.Document()

		cmd = ""
		if (value_type == 0):
			cmd = "select * from history where url like "
		elif (value_type == 1):
			cmd = "select * from history where tags like "

		cur = self.db_con.cursor()

		for s in values:
			search_string = "'%%%s%%'" % s
			cur.execute (cmd + search_string)

			element_bh = doc.createElement ("browserHistory")

			for (record_num,cache_filename,first_access_time,last_access_time,source,tags,title,url,visit_count) in cur:
				element_bhe = doc.createElement ("browserHistoryEntry")
		
				str_num = "%d" % record_num

				element_rec_num = doc.createElement ("recordNumber")
				rec_num_text = doc.createTextNode (str_num)
				element_rec_num.appendChild (rec_num_text)
				element_bhe.appendChild (element_rec_num)

				element_rec_num = doc.createElement ("cacheFilename")
				rec_num_text = doc.createTextNode (cache_filename)
				element_rec_num.appendChild (rec_num_text)
				element_bhe.appendChild (element_rec_num)

				element_rec_num = doc.createElement ("firstAccessTime")
				rec_num_text = doc.createTextNode (first_access_time)
				element_rec_num.appendChild (rec_num_text)
				element_bhe.appendChild (element_rec_num)

				element_rec_num = doc.createElement ("lastAccessTime")
				rec_num_text = doc.createTextNode (last_access_time)
				element_rec_num.appendChild (rec_num_text)
				element_bhe.appendChild (element_rec_num)

				element_rec_num = doc.createElement ("sourceFile")
				rec_num_text = doc.createTextNode (source)
				element_rec_num.appendChild (rec_num_text)
				element_bhe.appendChild (element_rec_num)

				element_rec_num = doc.createElement ("tags")
				rec_num_text = doc.createTextNode (tags)
				element_rec_num.appendChild (rec_num_text)
				element_bhe.appendChild (element_rec_num)

				element_rec_num = doc.createElement ("title")
				rec_num_text = doc.createTextNode (title)
				element_rec_num.appendChild (rec_num_text)
				element_bhe.appendChild (element_rec_num)

				element_rec_num = doc.createElement ("url")
				rec_num_text = doc.createTextNode (url)
				element_rec_num.appendChild (rec_num_text)
				element_bhe.appendChild (element_rec_num)

				str_num = "%s" % visit_count

				element_rec_num = doc.createElement ("visitCount")
				rec_num_text = doc.createTextNode (str_num)
				element_rec_num.appendChild (rec_num_text)
				element_bhe.appendChild (element_rec_num)

				element_bh.appendChild (element_bhe)

			doc.appendChild (element_bh)

		xml.dom.ext.PrettyPrint(doc, open(filename, "w"))	

	def on_export_all_activate (self, widget):
		dialog = gtk.FileChooserDialog("Save data as...",
		                               None,
		                               gtk.FILE_CHOOSER_ACTION_SAVE,
		                               (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
		                                gtk.STOCK_SAVE, gtk.RESPONSE_OK))

		dialog.set_default_response(gtk.RESPONSE_OK)

		filter = gtk.FileFilter()
		filter.set_name("Extensible Markup Language (XML)")
		filter.add_pattern("*.xml")
		dialog.add_filter(filter)

		filter = gtk.FileFilter()
		filter.set_name("Comma Seperated Value (CSV)")
		filter.add_pattern("*.csv")
		dialog.add_filter(filter)

		filter = gtk.FileFilter()
		filter.set_name("Tab Delimited")
		filter.add_pattern("*.txt")
		dialog.add_filter(filter)

		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			nf = dialog.get_filter().get_name()

			if (nf == "Comma Seperated Value (CSV)"):
				self.save_csv_history (dialog.get_filename())
			elif (nf == "Tab Delimited"):
				self.save_tab_history (dialog.get_filename())
			elif (nf == "Extensible Markup Language (XML)"):
				self.save_xml_history (dialog.get_filename())

		dialog.destroy()

	def on_export_by_host_activate (self, widget):
		dialog = exportDialog.ExportDialog (self.db_con, 0)
		hosts = dialog.get_user_selections ()

		if (len(hosts) == 0):
			return

		dialog = gtk.FileChooserDialog("Save data as...",
		                               None,
		                               gtk.FILE_CHOOSER_ACTION_SAVE,
		                               (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
		                                gtk.STOCK_SAVE, gtk.RESPONSE_OK))

		dialog.set_default_response(gtk.RESPONSE_OK)

		filter = gtk.FileFilter()
		filter.set_name("Extensible Markup Language (XML)")
		filter.add_pattern("*.xml")
		dialog.add_filter(filter)

		filter = gtk.FileFilter()
		filter.set_name("Comma Seperated Value (CSV)")
		filter.add_pattern("*.csv")
		dialog.add_filter(filter)

		filter = gtk.FileFilter()
		filter.set_name("Tab Delimited")
		filter.add_pattern("*.txt")
		dialog.add_filter(filter)

		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			nf = dialog.get_filter().get_name()

			if (nf == "Comma Seperated Value (CSV)"):
				self.save_csv_history_by_value (dialog.get_filename(), 0, hosts)
			elif (nf == "Tab Delimited"):
				self.save_tab_history_by_value (dialog.get_filename(), 0, hosts)
			elif (nf == "Extensible Markup Language (XML)"):
				self.save_xml_history_by_value (dialog.get_filename(), 0, hosts)

		dialog.destroy()

	def on_export_by_tag_activate (self, widget):
		dialog = exportDialog.ExportDialog (self.db_con, 1)
		tags = dialog.get_user_selections ()

		if (len(tags) == 0):
			return

		dialog = gtk.FileChooserDialog("Save data as...",
		                               None,
		                               gtk.FILE_CHOOSER_ACTION_SAVE,
		                               (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
		                                gtk.STOCK_SAVE, gtk.RESPONSE_OK))

		dialog.set_default_response(gtk.RESPONSE_OK)

		filter = gtk.FileFilter()
		filter.set_name("Extensible Markup Language (XML)")
		filter.add_pattern("*.xml")
		dialog.add_filter(filter)

		filter = gtk.FileFilter()
		filter.set_name("Comma Seperated Value (CSV)")
		filter.add_pattern("*.csv")
		dialog.add_filter(filter)

		filter = gtk.FileFilter()
		filter.set_name("Tab Delimited")
		filter.add_pattern("*.txt")
		dialog.add_filter(filter)

		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			nf = dialog.get_filter().get_name()

			if (nf == "Comma Seperated Value (CSV)"):
				self.save_csv_history_by_value (dialog.get_filename(), 1, tags)
			elif (nf == "Tab Delimited"):
				self.save_tab_history_by_value (dialog.get_filename(), 1, tags)
			elif (nf == "Extensible Markup Language (XML)"):
				self.save_xml_history_by_value (dialog.get_filename(), 1, tags)

		dialog.destroy()

	def on_close_activate (self, widget):
		if self.db_con == None:
			return

		self.db_con.close ()
		self.history_sources_store.clear ()
		self.history_sources_store.append (None, ['All history', 0])
		self.history_sources_treeview.set_sensitive (False)

		self.history_data_store.clear ()
		self.history_data_treeview.set_sensitive (False)

		self.moz.set_sensitive (False)

		self.toggle_search_view (False)
		self.toggle_toolbar (False)

		self.mainwindow.set_title ("Browser History Viewer")

	def on_preferences_activate (self, widget):
		message_box(None, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, "Not yet implemented.")

	def on_tag_activate (self, widget):
		model, paths = self.history_data_treeview.get_selection().get_selected_rows()
		if (paths == None):
			return

		dialog = applyTagDialog.ApplyTagDialog (self.db_con)
		tags = dialog.get_user_tags ()

		all_tags = ""
		for s in tags:
			all_tags += " %s " %s

		for p in paths:
			row = self.history_data_store.get_iter (p)
			record_num = self.history_data_store.get_value (row, 0)
		
			cur = self.db_con.cursor()
			cur.execute ("update history set tags=? where record_num=?", (all_tags.strip(), record_num))		

		self.db_con.commit ()
		self.on_history_source_activated (None, None, None)

	def on_tag_manager_activate (self, widget):
		dialog = tagDialog.TagManagerDialog (self.db_con)

	def file_history_go_to (self, path):
		if (os.path.isfile (path)):
			self.moz.load_url (path)
		elif (os.path.isdir (path)):
			self.directory_entry.set_text (path)
			self.current_dir = path
			self.update_file_list (self.current_dir)

	def on_back_activate (self, widget):
		try:
			page_num = self.notebook.get_current_page()
			if (page_num == 0):
				self.hv_history_index -= 1
				prev = self.hv_history [self.hv_history_index]
				self.moz.load_url (prev)
			elif (page_num == 1):
				self.file_history_index -= 1
				prev = self.file_history [self.file_history_index]
				self.file_history_go_to (prev)
		except:
			print "error on_back_activate"

	def on_forward_activate (self, widget):
		try:
			page_num = self.notebook.get_current_page()
			if (page_num == 0):
				self.hv_history_index += 1
				next = self.hv_history [self.hv_history_index]
				self.moz.load_url (next)
			if (page_num == 1):
				self.file_history_index += 1
				next = self.file_history [self.file_history_index]
				self.file_history_go_to (next)
		except:
			print "error on_foward_activate"

	def on_open_url (self, widget):
		url = ""
		(model, pathlist) = self.history_data_treeview.get_selection().get_selected_rows()
		for path in pathlist:
			url = model [path][7]
	
		gnome.url_show (url)

	def on_right_click (self, widget, event):
		if event.button == 3:
			popup = gtk.Menu ()

			item = gtk.MenuItem ("Open URL")
			item.connect ("activate", self.on_open_url)
			item.show ()

			popup.append (item)
			popup.popup (None, None, None, 3, event.time)

	def on_help_activated (self, widget):
		gnome.help_display_desktop(self.gnome_program, "bhv", "bhv.xml", "")

	def on_about_activated (self, widget):
		a = gtk.AboutDialog ()
		a.set_name (PACKAGE)
		a.set_version (VERSION)
		a.set_copyright ("Copyright (c) MMG Security, Inc.")
		a.set_comments ("Browser History Viewer")
		a.set_authors (['Loren Bandiera'])
		a.run ()
		a.destroy ()

def message_box(parent=None, flags=0, boxtype=gtk.MESSAGE_INFO, buttons=gtk.BUTTONS_NONE, message_format=None):
	dialog = gtk.MessageDialog(parent, flags, boxtype, buttons, message_format)
        response = dialog.run()
        dialog.destroy()

	return response
