#!/usr/bin/env python
#
# bhv - epiphany.py
# Author: Loren Bandiera
# Copyright 2006 MMG Security, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#

from xml.dom import minidom, Node
import time

class EpiphanyHistoryEntry:
	def __init__ (self):
		self.url = ""
		self.title = ""
		self.lastVisitedDate = ""
		self.firstVisitDate = ""
		self.visitCount = ""

class EpiphanyHistory:
	def __init__ (self, filename):
		self.browserHistory = []
		self.filename = filename

	def handle_node (self, node):
		ehe = EpiphanyHistoryEntry ()

		more_content = []

		for child in node.childNodes:

			content = []

			if (child.nodeName == "property" and child.nodeType == Node.ELEMENT_NODE):
				pid = child.attributes.get('id').value

				if (pid == "2"):					
					ehe.title = child.childNodes[0].nodeValue.strip()
				elif (pid == "3"):
					ehe.url = child.childNodes[0].nodeValue.strip()
				elif (pid == "4"):
					ehe.visitCount = child.childNodes[0].nodeValue			
				elif (pid == "5"):
					tmp = int (child.childNodes[0].nodeValue)
					ehe.lastVisitedDate = time.ctime (tmp)
				elif (pid == "6"):
					tmp = int (child.childNodes[0].nodeValue)
					ehe.firstVisitDate = time.ctime (tmp)

		self.browserHistory.append (ehe)

	def parse (self):
		xmldoc = minidom.parse (self.filename)
		history = xmldoc.getElementsByTagName ('node')

		for child in history:
			self.handle_node (child)
