#!/usr/bin/env python
#
# bhv - importDialog.py
# Author: Loren Bandiera
# Copyright 2005, 2006 MMG Security, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#

import sys
import gtk
import gtk.glade
import gobject
import threading
from config import *
from pysqlite2 import dbapi2 as sqlite
from xml.dom import minidom, Node
from time import sleep
import ie
import mozilla
import opera
import safari
import epiphany

class ImportDialog:
	def __init__ (self, db_connection):
		self.db_con = db_connection
		self.import_finished = True

		self.filename = ""
		self.description = ""
		self.current_file_type = 0
		self.cache_dir = ""

		self.glade_file = os.path.join (GLADEDIR, "browser-history-viewer.glade")
		self.Glade = gtk.glade.XML (self.glade_file, "importDialog")

		self.importDialog = self.Glade.get_widget ("importDialog")
		self.file_name_entry = self.Glade.get_widget ("fileNameEntry")
		self.description_entry = self.Glade.get_widget ("descriptionEntry")
		self.file_type_combo_box = self.Glade.get_widget ("fileTypeComboBox")
		self.progress_bar = self.Glade.get_widget ("importProgressBar")

		self.file_type_combo_box.set_active (0)

		# connect event handlers
		dic = { 
			"OnBrowseClicked" : self.on_browse_clicked,
			"OnOkClicked" : self.on_ok_clicked,
			"OnCancelClicked" : self.on_cancel_clicked
		}
		
		self.Glade.signal_autoconnect (dic)
		return

	def run (self):
		return self.importDialog.run ()

	def destroy (self):
		self.importDialog.destroy ()

	def get_file_name (self):
		return self.filename

	def get_file_type (self):
		return self.current_file_type

	def get_file_description (self):
		return self.description

	def check_file_type (self, filename):
		f = open (filename, "r")
		line = f.readline ()
		f.seek (0x18)
		version = f.read (4)
		f.close ()

		if (line.find ('<mdb:mork') >= 0):
			# mozilla
			return 1 
		elif (line.find ('<?xml') >= 0):
			xmldoc = minidom.parse (filename)
			ephy = xmldoc.getElementsByTagName ('ephy_history')
			if ephy is not None:
				# epiphany
				return 5

			saf = xmldoc.getElementsByTagName ('plist')
			if saf is not None:
				# safari
				return 4
		elif (line.startswith ('http')):
			# opera
			return 3 		
		elif (version == 4 or version >= 5):
			# internet explorer
			return 2 
 
		return 0;

	def on_browse_clicked (self, widget):
		dialog = gtk.FileChooserDialog("Select history file to import",
		                               None,
		                               gtk.FILE_CHOOSER_ACTION_OPEN,
		                               (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
		                                gtk.STOCK_OPEN, gtk.RESPONSE_OK))

		dialog.set_default_response(gtk.RESPONSE_OK)

		filter = gtk.FileFilter()
		filter.set_name("All files")
		filter.add_pattern("*")
		dialog.add_filter(filter)

		filter = gtk.FileFilter()
		filter.set_name("DAT files")
		filter.add_pattern("*.dat")
		dialog.add_filter(filter)

		filter = gtk.FileFilter()
		filter.set_name("plist files")
		filter.add_pattern("*.plist")
		dialog.add_filter(filter)

		filter = gtk.FileFilter()
		filter.set_name("XML files")
		filter.add_pattern("*.xml")
		dialog.add_filter(filter)

		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			filename = dialog.get_filename()
			self.cache_dir = dialog.get_current_folder_uri()
			self.file_name_entry.set_text (filename)

		dialog.destroy()

	def progress_timeout(self):
		if (self.import_finished == False):
			self.progress_bar.pulse ()

		return True

	def import_history (self):
		self.description = self.description_entry.get_text ()

		cur = self.db_con.cursor()
		cur.execute ("insert into source values (?, ?, ?)", (self.filename, self.description, self.current_file_type))
		i = 1

		# record num, cache_filename, first_access_time, last_access_time, source, tags, title, url, visit_count
		if (self.current_file_type == 1):
			self.mozilla = mozilla.MozillaHistory (self.filename)
			self.mozilla.parse ()

			for h in self.mozilla.browserHistory[:]:
				cur.execute ("insert into history values (?,'',?,?,?,'','',?,?)", (i, h.first_time, h.last_time, self.description, h.url, h.visit_count))
				i = i + 1

		elif (self.current_file_type == 2):
			self.ie = ie.IE (self.filename)
			self.ie.parse ()
		
			for h in self.ie.browserHistory[:]:
				cache_file = ""

				if h.dirname != "" and h.filename != "":
					cache_file = os.path.join (h.dirname, h.filename)

				cur.execute ("insert into history values (?,?,'',?,?,'','',?,0)", (i, cache_file, h.atime, self.description, h.url))
				i = i + 1

		elif (self.current_file_type == 3):
			self.opera = opera.Opera (self.filename)
			self.opera.parse ()

			for h in self.opera.browserHistory[:]:
				cur.execute ("insert into history values (?,'','',?,?,'',?,?,0)", (i, h.atime, self.description, h.title, h.url))
				i = i + 1

		elif (self.current_file_type == 4):
			self.safari  = safari.SafariHistory (self.filename)
			self.safari.parse ()
			for h in self.safari.browserHistory[:]:
				cur.execute ("insert into history values (?,'','',?,?,'',?,?,?)", (i, h.lastVisitedDate, self.description, h.title, h.url, h.visitCount))
				i = i + 1

		elif (self.current_file_type == 5):
			self.epiphany  = epiphany.EpiphanyHistory (self.filename)
			self.epiphany.parse ()
			for h in self.epiphany.browserHistory[:]:
				cur.execute ("insert into history values (?,'',?,?,?,'',?,?,?)", (i, h.firstVisitDate, h.lastVisitedDate, self.description, h.title, h.url, h.visitCount))
				i = i + 1

		self.db_con.commit ()
		self.import_finished = True

		gobject.source_remove (self.timer)
		self.timer = 0

	def on_ok_clicked (self, widget):
		self.filename = self.file_name_entry.get_text ()
		self.current_file_type = self.check_file_type (self.filename)

		self.import_finished = False
		self.timer = gobject.timeout_add (10, self.progress_timeout)

		thread = threading.Thread(target=self.import_history, args=([]))
		thread.start()

		while self.import_finished == False:
			while gtk.events_pending():
				gtk.main_iteration_do()

			sleep(0.1)

	def on_cancel_clicked (self, widget):
		print "canceled import"
