#!/usr/bin/env python
#
# bhv - mozilla.py
# Author: Loren Bandiera
# Copyright 2006 MMG Security, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#

import demork
import time

class MozillaHistoryEntry:
	def __init__ (self):
		self.first_time = ""
		self.last_time = ""
		self.visit_count = ""
		self.url = ""

class MozillaHistory:
	def __init__ (self, filename):
		self.browserHistory = []
		self.filename = filename

	def parse (self):
		self.db = demork.parse_mork (self.filename)
		self.handle_mork (self.db)

	def handle_mork (self, db):
		columns = db.cdict.keys()
		columns.sort(demork.hexcmp)

		tables = db.tables.keys()
		tables.sort(demork.hexcmp)

		for table in [ db.tables[k] for k in tables ]:
			rows = table.rows.keys()
			rows.sort(demork.hexcmp)

			for row in [ table.rows[k] for k in rows ]:
				mhe = MozillaHistoryEntry ()

				for cell in row.cells:
					if (cell.column == 'URL'):
						mhe.url = cell.atom
					elif (cell.column == 'FirstVisitDate'):
						tmp = int(cell.atom) / 1000000
						mhe.first_time = time.ctime (tmp)
					elif (cell.column == 'LastVisitDate'):
						tmp = int(cell.atom) / 1000000
						mhe.last_time = time.ctime (tmp)
					elif (cell.column == 'VisitCount'):
						mhe.visit_count = cell.atom

				self.browserHistory.append (mhe)
