#!/usr/bin/env python
#
# bhv - safari.py
# Author: Loren Bandiera
# Copyright 2006 MMG Security, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#

import time
from xml.dom import minidom, Node

class SafariHistoryEntry:
	def __init__ (self):
		self.url = ""
		self.title = ""
		self.lastVisitedDate = ""
		self.visitCount = ""

class SafariHistory:
	def __init__ (self, filename):
		self.browserHistory = []
		self.filename = filename

	def handle_node (self, node):
		she = SafariHistoryEntry ()

		more_content = []

		for child in node.childNodes:

			content = []

			if (child.nodeName == "key" and child.nodeType == Node.ELEMENT_NODE):
				for w in child.nextSibling.nextSibling.childNodes:
					content.append (w.nodeValue)

				more_content.append (content[0])

		size = len(more_content)

		if (size == 3):
			she.url = more_content[0]
			she.lastVisitedDate = more_content[1]
			she.visitCount = more_content[2]
		elif (size == 4):
			she.url = more_content[0]
			she.lastVisitedDate = more_content[1]
			she.title = more_content[2]
			she.visitCount = more_content[3]

		self.browserHistory.append (she)

	def parse (self):
		xmldoc = minidom.parse (self.filename)
		history = xmldoc.getElementsByTagName ('dict')

		index = 0

		for child in history:
			if (index == 0):
				index += 1
			else:
				self.handle_node (child)
