#!/usr/bin/env python
#
# bhv - tagDialog.py
# Author: Loren Bandiera
# Copyright 2005, 2006 MMG Security, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#

import sys
import gtk
import gtk.glade
import addTagDialog
from config import *
from pysqlite2 import dbapi2 as sqlite

class TagManagerDialog:
	def __init__ (self, db_connection):
		self.db_con = db_connection

		self.glade_file = os.path.join (GLADEDIR, "browser-history-viewer.glade")
		self.Glade = gtk.glade.XML (self.glade_file, "tagDialog")

		self.tagDialog = self.Glade.get_widget ("tagDialog")
		self.tag_treeview = self.Glade.get_widget ("tagTreeView")
		self.tag_store = gtk.ListStore (str, str)
		self.tag_treeview.set_model (self.tag_store)

		hc1 = self.create_tree_column ('Name', 0)
		self.tag_treeview.append_column (hc1)

		hc2 = self.create_tree_column ('Description', 1)
		self.tag_treeview.append_column (hc2)

		self.get_tags ()

		# connect event handlers
		dic = { 
			"OnAddClicked" : self.on_add_clicked,
			"OnRemoveClicked" : self.on_remove_clicked,
		}
		
		self.Glade.signal_autoconnect (dic)

		self.tagDialog.resize (320, 200)
		self.tagDialog.run ()
		self.tagDialog.destroy ()
		return

	def create_tree_column (self, name, pos):
		col = gtk.TreeViewColumn(name, gtk.CellRendererText(), text=pos)
		col.set_resizable (True)
		col.set_spacing (5)
		col.set_sort_column_id (pos)
		return col

	def get_tags (self):
		cur = self.db_con.cursor()
		cmd = "select * from tags"
		cur.execute (cmd)

		for (name,description) in cur:
			self.tag_store.append ([name, description])

	def on_add_clicked (self, widget):
		addTagDialog.AddTagDialog (self.db_con)
		self.tag_store.clear ()
		self.get_tags ()

	def on_remove_clicked (self, widget):
		model, row = self.tag_treeview.get_selection().get_selected()

		if (row == None):
			return

		name = self.tag_store.get_value (row, 0)

		cur = self.db_con.cursor()
		cur.execute ("delete from tags where name=?", (name,))
		self.db_con.commit()

		self.tag_store.clear ()
		self.get_tags ()
